@@ -37,7 +37,8 @@ logger = logging.getLogger(__name__)
 
 INSTRUMENT = {'avhrr/3': 'avhrr',
               'avhrr/2': 'avhrr',
-              'avhrr-3': 'avhrr'}
+              'avhrr-3': 'avhrr',
+              'mwhs-2': 'mwhs2'}
 
 
 class SwathBoundary(Boundary):
@@ -65,7 +66,7 @@ class SwathBoundary(Boundary):
         elif overpass.satellite == "noaa 16":
             scan_angle = 55.25
             instrument = "avhrr"
-        elif instrument == "mersi2":
+        elif instrument.startswith("mersi"):
             scan_angle = 55.4
             instrument = "avhrr"
         else:
@@ -78,10 +79,16 @@ class SwathBoundary(Boundary):
             sgeom = instrument_fun(scans_nb, scanpoints, scan_angle=scan_angle, frequency=100)
         elif instrument in ["ascat", ]:
             sgeom = instrument_fun(scans_nb, scanpoints)
+        elif instrument in ["amsua", 'mhs']:
+            sgeom = instrument_fun(scans_nb, scanpoints)
+        elif instrument in ["mwhs2", ]:
+            sgeom = instrument_fun(scans_nb, scanpoints)
         elif instrument in ["olci", ]:
             sgeom = instrument_fun(scans_nb, scanpoints)
         elif instrument == 'viirs':
             sgeom = instrument_fun(scans_nb, scanpoints, scan_step=scan_step)
+        elif instrument in ['mhs', 'atms', 'mwhs-2']:
+            sgeom = instrument_fun(scans_nb, scanpoints)
         else:
             logger.warning("Instrument not tested: %s", instrument)
             sgeom = instrument_fun(scans_nb)
@@ -99,7 +106,6 @@ class SwathBoundary(Boundary):
 
     def __init__(self, overpass, scan_step=50, frequency=200):
         # compute area covered by pass
-
         Boundary.__init__(self)
 
         self.overpass = overpass
@@ -115,34 +121,62 @@ class SwathBoundary(Boundary):
             sec_scan_duration = 1.779166667
             along_scan_reduce_factor = 1
         elif self.overpass.instrument.startswith("avhrr"):
-            sec_scan_duration = 1./6.
+            sec_scan_duration = 1. / 6.
             along_scan_reduce_factor = 0.1
         elif self.overpass.instrument == 'ascat':
             sec_scan_duration = 3.74747474747
             along_scan_reduce_factor = 1
             # Overwrite the scan step
             scan_step = 1
+        elif self.overpass.instrument == 'amsua':
+            sec_scan_duration = 8.
+            along_scan_reduce_factor = 1
+            # Overwrite the scan step
+            scan_step = 1
+        elif self.overpass.instrument == 'mhs':
+            sec_scan_duration = 8./3.
+            along_scan_reduce_factor = 1
+            # Overwrite the scan step
+            scan_step = 1
+        elif self.overpass.instrument == 'mwhs2':
+            sec_scan_duration = 8./3.
+            along_scan_reduce_factor = 1
+            # Overwrite the scan step
+            scan_step = 1
+        elif self.overpass.instrument == 'olci':
+            # 3 minutes of data is 4091 300meter lines:
+            sec_scan_duration = 0.04399902224395014
+            along_scan_reduce_factor = 1
+            # Overwrite the scan step
+            scan_step = 100
+        elif self.overpass.instrument == 'atms':
+            sec_scan_duration = 8/3.
+            along_scan_reduce_factor = 1
+            # Overwrite the scan step
+            scan_step = 1
+
         else:
             # Assume AVHRR!
             logmsg = ("Instrument scan duration not known. Setting it to AVHRR. Instrument: ")
             logger.info(logmsg + "%s", str(self.overpass.instrument))
-            sec_scan_duration = 1./6.
+            sec_scan_duration = 1. / 6.
             along_scan_reduce_factor = 0.1
 
         # From pass length in seconds and the seconds for one scan derive the number of scans in the swath:
-        scans_nb = scanlength_seconds/sec_scan_duration * along_scan_reduce_factor
+        scans_nb = scanlength_seconds / sec_scan_duration * along_scan_reduce_factor
         # Devide by the scan step to a reduced number of scans:
-        scans_nb = np.floor(scans_nb/scan_step)
+        scans_nb = np.floor(scans_nb / scan_step)
         scans_nb = int(max(scans_nb, 1))
 
         sides_lons, sides_lats = self.get_instrument_points(self.overpass,
                                                             overpass.risetime,
                                                             scans_nb,
-                                                            np.array([0, self.overpass.number_of_fovs-1]),
+                                                            np.array([0, self.overpass.number_of_fovs - 1]),
                                                             scan_step=scan_step)
 
         side_shape = sides_lons[::-1, 0].shape[0]
         nmod = 1
+
         if side_shape != scans_nb:
             nmod = side_shape // scans_nb
             logger.debug('Number of scan lines (%d) does not match number of scans (%d)',
@@ -182,6 +216,8 @@ class SwathBoundary(Boundary):
         self.top_lons = lons[0]
         self.top_lats = lats[0]
 
+        return
+
     def decimate(self, ratio):
         l = len(self.top_lons)
         start = (l % ratio) / 2
@@ -201,6 +237,8 @@ class SwathBoundary(Boundary):
         self.left_lons = self.left_lons[points]
         self.left_lats = self.left_lats[points]
 
+        return
+
     def contour(self):
         lons = np.concatenate((self.top_lons,
                                self.right_lons[1:-1],
