@@ -29,27 +29,25 @@ import logging
 import logging.handlers
 import operator
 import os
-import six
 import socket
-from functools import reduce as fctools_reduce
-try:
-    from urllib.parse import urlparse
-except ImportError:
-    from urlparse import urlparse
-
+import sys
 from datetime import datetime, timedelta
+from functools import reduce as fctools_reduce
 from tempfile import mkstemp
+
 import numpy as np
+from urllib.parse import urlparse
 
 from pyorbital import orbital, tlefile
 from pyresample.boundary import AreaDefBoundary
+from trollsched import MIN_PASS, NOAA20_NAME, NUMBER_OF_FOVS
 from trollsched.boundary import SwathBoundary
-from trollsched import (MIN_PASS, NOAA20_NAME, NUMBER_OF_FOVS)
 
 logger = logging.getLogger(__name__)
 
 VIIRS_PLATFORM_NAMES = ['SUOMI NPP', 'SNPP',
                         'NOAA-20', 'NOAA 20']
+MERSI_PLATFORM_NAMES = ['FENGYUN 3C', 'FENGYUN-3C', 'FY-3C']
 MERSI2_PLATFORM_NAMES = ['FENGYUN 3D', 'FENGYUN-3D', 'FY-3D',
                          'FENGYUN 3E', 'FENGYUN-3E', 'FY-3E']
 
@@ -147,7 +145,7 @@ class Pass(SimplePass):
 
         if isinstance(instrument, (list, set)):
             if 'avhrr' in instrument:
-                logger.warning("Instrument is a sequence Assume avhrr...")
+                logger.warning("Instrument is a sequence! Assume avhrr...")
                 instrument = 'avhrr'
             elif 'viirs' in instrument:
                 logger.warning("Instrument is a sequence! Assume viirs...")
@@ -155,6 +153,12 @@ class Pass(SimplePass):
             elif 'modis' in instrument:
                 logger.warning("Instrument is a sequence! Assume modis...")
                 instrument = 'modis'
+            elif 'mersi' in instrument:
+                logger.warning("Instrument is a sequence! Assume mersi...")
+                instrument = 'mersi'
+            elif 'mersi-2' in instrument:
+                logger.warning("Instrument is a sequence! Assume mersi-2...")
+                instrument = 'mersi-2'
             else:
                 raise TypeError("Instrument is a sequence! Don't know which one to choose!")
 
@@ -245,6 +249,107 @@ class Pass(SimplePass):
             return 0
         return inter.area() / area_boundary.area()
 
+    def generate_metno_xml(self, coords, root):
+        import xml.etree.ElementTree as ET
+
+        asimuth_at_max_elevation, max_elevation = self.orb.get_observer_look(self.uptime, *coords)
+        pass_direction = self.pass_direction().capitalize()[:1]
+        # anl = self.orb.get_lonlatalt(self.orb.get_last_an_time(self.risetime))[0] % 360
+        asimuth_at_aos, aos_elevation = self.orb.get_observer_look(self.risetime, *coords)
+        orbit = self.orb.get_orbit_number(self.risetime)
+        # aos_epoch=int((self.risetime-datetime(1970,1,1)).total_seconds())
+        sat_lon, sat_lat, alt = self.orb.get_lonlatalt(self.risetime)
+
+        ovpass = ET.SubElement(root, "pass")
+        ovpass.set("satellite", self.satellite.name)
+        ovpass.set("aos", self.risetime.strftime("%Y%m%d%H%M%S"))
+        ovpass.set("los", self.falltime.strftime("%Y%m%d%H%M%S"))
+        ovpass.set("orbit", "{:d}".format(orbit))
+        ovpass.set("max-elevation", "{:.3f}".format(max_elevation))
+        ovpass.set("asimuth-at-max-elevation", "{:.3f}".format(asimuth_at_max_elevation))
+        ovpass.set("asimuth-at-aos", "{:.3f}".format(asimuth_at_aos))
+        ovpass.set("pass-direction", pass_direction)
+        ovpass.set("satellite-lon-at-aos", "{:.3f}".format(sat_lon))
+        ovpass.set("satellite-lat-at-aos", "{:.3f}".format(sat_lat))
+        ovpass.set("tle-epoch", self.orb.orbit_elements.epoch.astype(datetime).strftime("%Y%m%d%H%M%S.%f"))
+        if self.fig:
+            ovpass.set("figure", self.fig)
+
+        return True
+
+    def print_meos(self, coords, line_no):
+        """
+         No. Date    Satellite  Orbit Max EL  AOS      Ovlp  LOS      Durtn  Az(AOS/MAX)
+        """
+
+        asimuth_at_max_elevation, max_elevation = self.orb.get_observer_look(self.uptime, *coords)
+        pass_direction = self.pass_direction().capitalize()[:1]
+        # anl = self.orb.get_lonlatalt(self.orb.get_last_an_time(self.risetime))[0] % 360
+        asimuth_at_aos, aos_elevation = self.orb.get_observer_look(self.risetime, *coords)
+        orbit = self.orb.get_orbit_number(self.risetime)
+        aos_epoch = int((self.risetime - datetime(1970, 1, 1)).total_seconds())
+        sat_lon, sat_lat, alt = self.orb.get_lonlatalt(self.risetime)
+
+        dur_secs = (self.falltime - self.risetime).seconds
+        dur_hours, dur_reminder = divmod(dur_secs, 3600)
+        dur_minutes, dur_seconds = divmod(dur_reminder, 60)
+        duration = "{:0>2}:{:0>2}".format(dur_minutes, dur_seconds)
+
+        satellite_meos_translation = {"NOAA 19": "NOAA_19",
+                                      "NOAA 18": "NOAA_18",
+                                      "NOAA 15": "NOAA_15",
+                                      "METOP-A": "M02",
+                                      "METOP-B": "M01",
+                                      "FENGYUN 3A": "FENGYUN-3A",
+                                      "FENGYUN 3B": "FENGYUN-3B",
+                                      "FENGYUN 3C": "FENGYUN-3C",
+                                      "SUOMI NPP": "NPP"}
+
+        import hashlib
+
+        pass_key = hashlib.md5(("{:s}|{:d}|{:d}|{:.3f}|{:.3f}".
+                                format(satellite_meos_translation.get(self.satellite.name.upper(),
+                                                                      self.satellite.name.upper()),
+                                       int(orbit),
+                                       aos_epoch,
+                                       sat_lon,
+                                       sat_lat)).encode('utf-8')).hexdigest()
+
+        line_list = [" {line_no:>2}",
+                     "{date}",
+                     "{satellite:<10}",
+                     "{orbit:>5}",
+                     "{elevation:>6.3f} ",
+                     "{risetime}",
+                     "{overlap:<5s}",
+                     "{falltime}",
+                     "{duration}",
+                     "{asimuth_at_aos:>5.1f}",
+                     "{asimuth_at_max:>5.1f}",
+                     "-- Undefined(Scheduling not done {aos_epoch} )",
+                     "{passkey}",
+                     "{pass_direction}"
+                     ]
+
+        line = " ".join(line_list).format(
+            # line_no=line_no,
+            line_no=1,
+            date=self.risetime.strftime("%Y%m%d"),
+            satellite=satellite_meos_translation.get(self.satellite.name.upper(),
+                                                     self.satellite.name.upper()),
+            orbit=orbit,
+            elevation=max_elevation,
+            risetime=self.risetime.strftime("%H:%M:%S"),
+            overlap="n/a",
+            falltime=self.falltime.strftime("%H:%M:%S"),
+            duration=duration,
+            asimuth_at_aos=asimuth_at_aos,
+            asimuth_at_max=asimuth_at_max_elevation,
+            aos_epoch=aos_epoch,
+            passkey=pass_key,
+            pass_direction=pass_direction)
+        return line
+
     def print_vcs(self, coords):
         """Should look like this::
 
@@ -327,13 +432,13 @@ def get_aqua_terra_dumpdata_from_ftp(sat, dump_url):
     """
 
     logger.info("Fetch %s dump info from internet", str(sat.name))
-    if isinstance(dump_url, six.text_type):
+    if isinstance(dump_url, str):
         url = urlparse(dump_url % sat.name)
     else:
         url = urlparse(HOST % sat.name)
     logger.debug("Connect to ftp server")
     try:
-        f = ftplib.FTP(url.netloc)
+        f = ftplib.FTP_TLS(url.netloc)
     except (socket.error, socket.gaierror) as e:
         logger.error('cannot reach to %s ' % HOST + str(e))
         f = None
@@ -350,6 +455,7 @@ def get_aqua_terra_dumpdata_from_ftp(sat, dump_url):
     if f is not None:
         data = []
         try:
+            f.prot_p()  # explicitly call for protected transfer
             f.dir(url.path, data.append)
         except socket.error as e:
             logger.error("Can't get any data: " + str(e))
@@ -377,6 +483,7 @@ def get_aqua_terra_dumpdata_from_ftp(sat, dump_url):
         lines = []
         if not os.path.exists(os.path.join("/tmp", filedates[date])):
             try:
+                f.prot_p()  # explicitly call for protected transfer
                 f.retrlines('RETR ' + os.path.join(url.path, filedates[date]),
                             lines.append)
             except ftplib.error_perm:
@@ -422,7 +529,9 @@ def get_next_passes(satellites,
                     forward,
                     coords,
                     tle_file=None,
-                    aqua_terra_dumps=None):
+                    aqua_terra_dumps=None,
+                    min_pass=MIN_PASS,
+                    local_horizon=0):
     """Get the next passes for *satellites*, starting at *utctime*, for a
     duration of *forward* hours, with observer at *coords* ie lon (°E), lat
     (°N), altitude (km). Uses *tle_file* if provided, downloads from celestrack
@@ -448,24 +557,30 @@ def get_next_passes(satellites,
             sat = Satellite(sat, 0, 0)
 
         satorb = orbital.Orbital(sat.name, tle_file=tle_file)
-        passlist = satorb.get_next_passes(utctime, forward, *coords)
+        passlist = satorb.get_next_passes(utctime,
+                                          forward,
+                                          horizon=local_horizon,
+                                          *coords
+                                          )
 
-        if sat.name == "metop-a":
-            # Take care of metop-a
+        if sat.name.lower() == "metop-a":
+            # Take care of metop-a special case
             passes["metop-a"] = get_metopa_passes(sat, passlist, satorb)
-
-        elif sat.name in ["aqua", "terra"] and aqua_terra_dumps:
+        elif sat.name.lower() in ["aqua", "terra"] and aqua_terra_dumps:
             # Take care of aqua (dumps in svalbard and poker flat)
             # Get the Terra/Aqua passes and fill the passes dict:
             get_terra_aqua_passes(passes, utctime, forward, sat, passlist, satorb, aqua_terra_dumps)
-
         else:
             if sat.name.upper() in VIIRS_PLATFORM_NAMES:
                 instrument = "viirs"
             elif sat.name.lower().startswith("metop") or sat.name.lower().startswith("noaa"):
                 instrument = "avhrr"
+            elif sat.name.lower() in ["aqua", "terra"]:  # when aqua_terra_dumps=False
+                instrument = "modis"
+            elif sat.name.upper() in MERSI_PLATFORM_NAMES:
+                instrument = "mersi"
             elif sat.name.upper() in MERSI2_PLATFORM_NAMES:
-                instrument = "mersi2"
+                instrument = "mersi-2"
             else:
                 instrument = "unknown"
 
