@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2018 - 2019 PyTroll
+# Copyright (c) 2018 - 2021 Pytroll-schedule developers
 
 # Author(s):
 
@@ -29,7 +29,7 @@ from datetime import datetime, timedelta
 from trollsched.satpass import Pass
 from trollsched.boundary import SwathBoundary
 from pyorbital.orbital import Orbital
-
+from pyresample.geometry import AreaDefinition, create_area_def
 
 LONS1 = np.array([-122.29913729160562, -131.54385362589042, -155.788034272281,
                   143.1730880418349, 105.69172088208997, 93.03135571771092,
@@ -128,6 +128,11 @@ LATS3 = np.array([66.94713585, 67.07854554, 66.53108388, 65.27837805, 63.5022359
                   58.33858588, 57.71210872, 55.14964148, 55.72506407, 60.40889798,
                   61.99561474, 63.11425455, 63.67173255, 63.56939058], dtype='float64')
 
+AREA_DEF_EURON1 = AreaDefinition('euron1', 'Northern Europe - 1km',
+                                 '', {'proj': 'stere', 'ellps': 'WGS84',
+                                      'lat_0': 90.0, 'lon_0': 0.0, 'lat_ts': 60.0},
+                                 3072, 3072, (-1000000.0, -4500000.0, 2072000.0, -1428000.0))
+
 
 def assertNumpyArraysEqual(self, other):
     if self.shape != other.shape:
@@ -154,13 +159,14 @@ def get_n19_orbital():
     return Orbital('NOAA-19', line1=tle1, line2=tle2)
 
 
-def get_region(areaid):
-    try:
-        from satpy.resample import get_area_def
-    except ImportError:
-        from mpop.projector import get_area_def
+def get_mb_orbital():
+    """Return orbital for a given set of TLEs for MetOp-B.
 
-    return get_area_def(areaid)
+    From 2021-02-04
+    """
+    tle1 = "1 38771U 12049A   21034.58230818 -.00000012  00000-0  14602-4 0 9998"
+    tle2 = "2 38771  98.6992  96.5537 0002329  71.3979  35.1836 14.21496632434867"
+    return Orbital("Metop-B", line1=tle1, line2=tle2)
 
 
 class TestPass(unittest.TestCase):
@@ -175,6 +181,11 @@ class TestPass(unittest.TestCase):
         tstart = datetime(2018, 10, 16, 2, 48, 29)
         tend = datetime(2018, 10, 16, 3, 2, 38)
 
+        instruments = set(('viirs', 'avhrr', 'modis', 'mersi', 'mersi-2'))
+        for instrument in instruments:
+            overp = Pass('NOAA-20', tstart, tend, orb=self.n20orb, instrument=instrument)
+            self.assertEqual(overp.instrument, instrument)
+
         instruments = set(('viirs', 'avhrr', 'modis'))
         overp = Pass('NOAA-20', tstart, tend, orb=self.n20orb, instrument=instruments)
         self.assertEqual(overp.instrument, 'avhrr')
@@ -198,7 +209,24 @@ class TestSwathBoundary(unittest.TestCase):
         """Set up"""
         self.n20orb = get_n20_orbital()
         self.n19orb = get_n19_orbital()
-        self.euron1 = get_region('euron1')
+        self.mborb = get_mb_orbital()
+        self.euron1 = AREA_DEF_EURON1
+        self.antarctica = create_area_def(
+            "antarctic",
+            {'ellps': 'WGS84', 'lat_0': '-90', 'lat_ts': '-60',
+             'lon_0': '0', 'no_defs': 'None', 'proj': 'stere',
+             'type': 'crs', 'units': 'm', 'x_0': '0', 'y_0': '0'},
+            width=1000, height=1000,
+            area_extent=(-4008875.4031, -4000855.294,
+                         4000855.9937, 4008874.7048))
+        self.arctica = create_area_def(
+            "arctic",
+            {'ellps': 'WGS84', 'lat_0': '90', 'lat_ts': '60',
+             'lon_0': '0', 'no_defs': 'None', 'proj': 'stere',
+             'type': 'crs', 'units': 'm', 'x_0': '0', 'y_0': '0'},
+            width=1000, height=1000,
+            area_extent=(-4008875.4031, -4000855.294,
+                         4000855.9937, 4008874.7048))
 
     def test_swath_boundary(self):
 
@@ -313,9 +341,80 @@ class TestSwathBoundary(unittest.TestCase):
 
         mypass = Pass('FENGYUN 3D', tstart, tend, instrument='mersi2', tle1=tle1, tle2=tle2)
         cov = mypass.area_coverage(self.euron1)
-
         self.assertAlmostEqual(cov, 0.786836, 5)
 
+        mypass = Pass('FENGYUN 3D', tstart, tend, instrument='mersi-2', tle1=tle1, tle2=tle2)
+        cov = mypass.area_coverage(self.euron1)
+        self.assertAlmostEqual(cov, 0.786836, 5)
+
+    def test_arctic_is_not_antarctic(self):
+
+        tstart = datetime(2021, 2, 3, 16, 28, 3)
+        tend = datetime(2021, 2, 3, 16, 31, 3)
+
+        overp = Pass('Metop-B', tstart, tend, orb=self.mborb, instrument='avhrr')
+
+        cov_south = overp.area_coverage(self.antarctica)
+        cov_north = overp.area_coverage(self.arctica)
+
+        assert cov_north == 0
+        assert cov_south != 0
+
+    def tearDown(self):
+        """Clean up"""
+        pass
+
+
+class TestPassList(unittest.TestCase):
+
+    def setUp(self):
+        """Set up"""
+        pass
+
+    def test_meos_pass_list(self):
+        orig = ("  1 20190105 FENGYUN 3D  5907 52.943  01:01:45 n/a   01:17:15 15:30  18.6 107.4 -- "
+                "Undefined(Scheduling not done 1546650105 ) a3d0df0cd289244e2f39f613f229a5cc D")
+
+        tstart = datetime.strptime("2019-01-05T01:01:45", "%Y-%m-%dT%H:%M:%S")
+        tend = tstart + timedelta(seconds=60 * 15.5)
+
+        tle1 = '1 43010U 17072A   18363.54078832 -.00000045  00000-0 -79715-6 0  9999'
+        tle2 = '2 43010  98.6971 300.6571 0001567 143.5989 216.5282 14.19710974 58158'
+
+        mypass = Pass('FENGYUN 3D', tstart, tend, instrument='mersi2', tle1=tle1, tle2=tle2)
+        coords = (10.72, 59.942, 0.1)
+        meos_format_str = mypass.print_meos(coords, line_no=1)
+        self.assertEqual(meos_format_str, orig)
+
+        mypass = Pass('FENGYUN 3D', tstart, tend, instrument='mersi-2', tle1=tle1, tle2=tle2)
+        coords = (10.72, 59.942, 0.1)
+        meos_format_str = mypass.print_meos(coords, line_no=1)
+        self.assertEqual(meos_format_str, orig)
+
+    def test_generate_metno_xml(self):
+        import xml.etree.ElementTree as ET
+        root = ET.Element("acquisition-schedule")
+
+        orig = ('<acquisition-schedule><pass satellite="FENGYUN 3D" aos="20190105010145" los="20190105011715" '
+                'orbit="5907" max-elevation="52.943" asimuth-at-max-elevation="107.385" asimuth-at-aos="18.555" '
+                'pass-direction="D" satellite-lon-at-aos="76.204" satellite-lat-at-aos="80.739" '
+                'tle-epoch="20181229125844.110848" /></acquisition-schedule>')
+
+        tstart = datetime.strptime("2019-01-05T01:01:45", "%Y-%m-%dT%H:%M:%S")
+        tend = tstart + timedelta(seconds=60 * 15.5)
+
+        tle1 = '1 43010U 17072A   18363.54078832 -.00000045  00000-0 -79715-6 0  9999'
+        tle2 = '2 43010  98.6971 300.6571 0001567 143.5989 216.5282 14.19710974 58158'
+
+        mypass = Pass('FENGYUN 3D', tstart, tend, instrument='mersi2', tle1=tle1, tle2=tle2)
+
+        coords = (10.72, 59.942, 0.1)
+        mypass.generate_metno_xml(coords, root)
+
+        # Dictionaries don't have guaranteed ordering in Python 3.7, so convert the strings to sets and compare them
+        res = set(ET.tostring(root).decode("utf-8").split())
+        self.assertEqual(res, set(orig.split()))
+
     def tearDown(self):
         """Clean up"""
         pass
@@ -328,5 +427,6 @@ def suite():
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(TestSwathBoundary))
     mysuite.addTest(loader.loadTestsFromTestCase(TestPass))
+    mysuite.addTest(loader.loadTestsFromTestCase(TestPassList))
 
     return mysuite
