@@ -35,15 +35,9 @@ from trollsched.satpass import get_aqua_terra_dumps
 from trollsched.satpass import get_metopa_passes
 
 import sys
-if sys.version_info < (2, 7):
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
+from unittest.mock import patch
 
 
 # class TestPass(unittest.TestCase):
@@ -302,26 +296,20 @@ class TestAll(unittest.TestCase):
 
             self.assertEqual(len(allpasses), 2)
 
-            n20pass1 = allpasses.pop()
-
             rt1 = datetime(2018, 11, 28, 10, 53, 42, 79483)
             ft1 = datetime(2018, 11, 28, 11, 9, 6, 916787)
             rt2 = datetime(2018, 11, 28, 12, 34, 44, 667963)
             ft2 = datetime(2018, 11, 28, 12, 49, 25, 134067)
 
-            dt_ = n20pass1.risetime - rt1
-            self.assertAlmostEqual(dt_.seconds, 0)
+            rise_times = [p.risetime for p in allpasses]
+            fall_times = [p.falltime for p in allpasses]
 
-            dt_ = n20pass1.falltime - ft1
-            self.assertAlmostEqual(dt_.seconds, 0)
+            assert rt1 in rise_times
+            assert rt2 in rise_times
+            assert ft1 in fall_times
+            assert ft2 in fall_times
 
-            n20pass2 = allpasses.pop()
-
-            dt_ = n20pass2.risetime - rt2
-            self.assertAlmostEqual(dt_.seconds, 0)
-
-            dt_ = n20pass2.falltime - ft2
-            self.assertAlmostEqual(dt_.seconds, 0)
+            assert all([p.instrument == 'viirs' for p in allpasses])
 
     @patch('os.path.exists')
     @patch('trollsched.satpass.get_aqua_terra_dumpdata_from_ftp')
@@ -364,6 +352,8 @@ class TestAll(unittest.TestCase):
 
                 self.assertAlmostEqual(dtmin.seconds, 0)
 
+                self.assertEqual(mypass.instrument, 'modis')
+
     @patch('trollsched.satpass.get_aqua_terra_dumpdata_from_ftp')
     def test_get_aqua_terra_dumps(self, dumps_from_ftp):
         dumps_from_ftp.return_value = self.dumpdata_terra
@@ -405,9 +395,9 @@ class TestAll(unittest.TestCase):
 
             self.assertEqual(len(metopa_passes), 2)
             self.assertEqual(metopa_passes[0].pass_direction(), 'descending')
-            self.assertEqual(metopa_passes[0].seconds(), 462.466119)
+            self.assertAlmostEqual(metopa_passes[0].seconds(), 487.512589, 5)
             self.assertEqual((metopa_passes[0].uptime - datetime(2018, 12, 4, 9, 17, 48, 530484)).seconds, 0)
-            self.assertEqual((metopa_passes[0].risetime - datetime(2018, 12, 4, 9, 17, 46, 691075)).seconds, 0)
+            self.assertEqual((metopa_passes[0].risetime - datetime(2018, 12, 4, 9, 17, 21, 644605)).seconds, 0)
 
     def tearDown(self):
         """Clean up"""
